import java.awt.Color;
/**
 * A plane in 3-dimensional space.
 * 
 * @author Duta
 * @version 9/3/2013
 */
public class Plane extends WorldObject {
    private Vector point;
    private Vector normal;
    
    public Plane(Vector point, Vector normal, Color color) {
        super(color);
        this.point = point;
        this.normal = normal;
    }
    
    public Intersection getIntersection(Ray ray) {
        double d = ray.getDirection().dotProduct(normal);
        // If 0, then the ray and
        // the plane are parallel.
        if(d == 0) {
            // Therefore there is
            // no intersection.
            return new Intersection(
                new Vector(0, 0, 0),
                -1,
                new Vector(0, 0, 0),
                this);
        } else {
            // Otherwise, return the
            // point of intersection.
            d = point.subtract(ray.getOrigin()).dotProduct(normal)/d;
            return new Intersection(
                ray.getOrigin().add(ray.getDirection().multiply(d)),
                d,
                normal,
                this);
        }
    }
    
    public Vector getPoint() {
        return point;
    }
    
    public Vector getNormal() {
        return normal;
    }
}
