import java.awt.Color;
/**
 * Used for representing a sphere.
 * 
 * @author Duta
 * @version 9/3/2013
 */
public class Sphere extends WorldObject {
    private Vector center;
    private double radius;
    
    public Sphere(Vector center, double radius, Color color) {
        super(color);
        this.center = center;
        this.radius = radius;
    }
    
    public Intersection getIntersection(Ray ray) {
        Vector v = ray.getOrigin().subtract(center);
        double a = ray.getDirection().dotProduct(v);
        double b = v.dotProduct(v);
        double q = a*a - b + radius*radius;
        if(q < 0) {
            // If q is negative, then
            // there is no intersection.
            return new Intersection(
                new Vector(0, 0, 0),
                -1,
                new Vector(0, 0, 0),
                this);
        } else {
            q = Math.sqrt(q);
            // Find the two roots (-a +/- q)
            double d1 = -a - q;
            double d2 = -a + q;
            double d3;
            // Work out which one is
            // the correct intersection.
            if(d1 > 0 && (d1 < d2 || d2 < 0)) {
                d3 = d1;
            } else if(d2 > 0 && (d2 < d1 || d1 < 0)) {
                d3 = d2;
            } else {
                // No intersection
                return new Intersection(
                    new Vector(0, 0, 0),
                    -1,
                    new Vector(0, 0, 0),
                    this);
            }
            Vector v2 = ray.getOrigin().add(ray.getDirection().multiply(d3));
            return new Intersection(
                v2,
                d3,
                getNormal(v2),
                this);
        }
    }
    
    public Vector getCenter() {
        return center;
    }
    
    public double getRadius() {
        return radius;
    }
    
    private Vector getNormal(Vector v) {
        return v.subtract(center).getNormal();
    }
}
