/**
 * Used for representing 3-dimensional vectors.
 * 
 * @author Duta
 * @version 9/3/2013
 */
public class Vector {
    private double x, y, z;

    public Vector(double x, double y, double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public double dotProduct(Vector v) {
        return x*v.x + y*v.y + z*v.z;
    }
    
    public Vector crossProduct(Vector v) {
        return new Vector(
            y*v.z - z*v.y,
            z*v.x - x*v.z,
            x*v.y - y*v.x);
    }
    
    public double getMagnitude() {
        return Math.sqrt(x*x + y*y + z*z);
    }
    
    public Vector getNormal() {
        double magnitude = getMagnitude();
        return new Vector(
            x/magnitude,
            y/magnitude,
            z/magnitude);
    }
    
    public Vector add(Vector v) {
        return new Vector(
            x + v.x,
            y + v.y,
            z + v.z);
    }
    
    public Vector subtract(Vector v) {
        return new Vector(
            x - v.x,
            y - v.y,
            z - v.z);
    }
    
    public Vector multiply(double sf) {
        return new Vector(
            x*sf,
            y*sf,
            z*sf);
    }
    
    public double getX() {
        return x;
    }
    
    public double getY() {
        return y;
    }
    
    public double getZ() {
        return z;
    }
}






























